CREATE PROCEDURE INIT_LoginDetect (
	@sPrintMe		CHAR(1),
	@sOwnTrans		CHAR(1),

	/* other vars with descriptions */
	@sLogin				CHAR(255) = '',
	@sPassword			CHAR(255) = '',
	@iDomainName		INT = 0,
	@sGroupType			CHAR(1) = '' OUTPUT,
	@sGlobalRotations	INT = 0 OUTPUT,
	@sGlobalDateBegin	SMALLDATETIME OUTPUT,
	@sGlobalGroupUnq	INT = 0 OUTPUT,
	@sGlobalPosition	INT = 0 OUTPUT,
	@iGlobalNumDays		INT = 0 OUTPUT,
	@iLoginAccountUnq	INT = -1 OUTPUT,
	@bHasAccount		CHAR(1) = 'N' OUTPUT
)
WITH ENCRYPTION
AS

/* Additional variables used in this procedure */
DECLARE
	@sIntransaction  	CHAR(1),		/* Indicate finalization routine to commit or rollback */
	@iRtnCode    		INT,			/* Return 0 for success, prefer positive values for failure */
	@bFirstCheck		INT,
	@ALLDOMAINS			INT,
	@HOMEONLY			INT


/* Initialize the main variables in use */
SELECT @iRtnCode = 0, @sInTransaction = 'N', @bFirstCheck = 1, @ALLDOMAINS = -2, @HOMEONLY = -1
SELECT @iLoginAccountUnq = NULL

IF @sLogin = '' BEGIN
	GOTO Proc_Finalize
END

IF @sPassword = '' BEGIN
	GOTO Proc_Finalize
END

IF @iDomainName = 0 BEGIN
	GOTO Proc_Finalize
END

/* Start the transaction if appropriate */
IF @sOwnTrans = 'Y' BEGIN
	BEGIN TRANSACTION
	SELECT @sInTransaction = 'Y'
END

/* Check to see if AccountDomainMap has an entry for either ALLDOMAINS(-2) or the domain they are currently on - if so, log them in */
SELECT @iLoginAccountUnq = A.AccountUnq FROM Accounts A (NOLOCK), AccountDomainMap M (NOLOCK) WHERE 
				A.Login = @sLogin AND 
				A.Password = @sPassword AND 
				A.AccountUnq = M.AccountUnq AND 
					(M.DomainUnq = @iDomainName OR 
						M.DomainUnq = @ALLDOMAINS) AND 
					(A.AddDate > A.RemoveDate OR 
						A.RemoveDate IS NULL)
IF NOT (@iLoginAccountUnq IS NULL)
BEGIN
	/* They can log onto this domain -- but double check to make sure they do not have a HOMEONLY entry too! */
	/* See if they have a HOMEONLY(-1) entry. If so, the current domain must be their HomeDomain or don't log them in */
	SELECT @iLoginAccountUnq = A.AccountUnq FROM Accounts A (NOLOCK), AccountDomainMap M (NOLOCK) WHERE 
					A.Login = @sLogin AND 
					A.Password = @sPassword AND
					A.AccountUnq = M.AccountUnq AND 
					M.DomainUnq = @HOMEONLY AND 
						(A.AddDate > A.RemoveDate OR 
							A.RemoveDate IS NULL)
	IF NOT (@iLoginAccountUnq IS NULL)
	BEGIN
		/* Since they have a HOMEONLY entry, we must now must check if the current domain is their Home Domain */
		SELECT @bFirstCheck = 0
	END ELSE
	BEGIN
		/* They do not have a HOMEONLY entry so the previous answer is ok - log them in & get any Group info they might have */
		SELECT @bHasAccount = 'Y'
		SELECT DISTINCT 
			@sGlobalGroupUnq = G.GroupUnq, 
			@sGroupType = G.Type, 
			@sGlobalRotations = G.Rotations, 
			@sGlobalDateBegin = G.DateBegin, 
			@iGlobalNumDays = M.NumDays, 
			@sGlobalPosition = M.Position 
			FROM GroupMap M (NOLOCK), Groups G (NOLOCK) WHERE 
				M.AccountUnq = @iLoginAccountUnq AND 
				M.GroupUnq = G.GroupUnq AND 
				G.DomainUnq = @iDomainName AND 
				G.DomainUnq = M.DomainUnq
		GOTO Proc_Finalize
	END
END ELSE
BEGIN
	/* They have no ALLDOMAINS or domain entry for this domain -- they must have an entry for their Home Domain or they can't log in */
	SELECT @bFirstCheck = 0
END



IF ( @bFirstCheck = 0 )
BEGIN
	/* Since they do not have an ALLDOMAINS entry, or entry for this Domain, OR they have a HOMEONLY entry, we must check */
	/* 	to see if the current Domain is their Home Domain - if not, do not log them in. */
	SELECT @iLoginAccountUnq = AccountUnq FROM Accounts (NOLOCK) WHERE 
						Login = @sLogin AND 
						Password = @sPassword AND 
						HomeDomain = @iDomainName AND 
							(AddDate > RemoveDate OR 
								RemoveDate IS NULL)
	IF NOT (@iLoginAccountUnq IS NULL)
	BEGIN
		/* The current domain is their Home Domain - log them in and find and Group info they might have */
		SELECT @bHasAccount = 'Y'
		SELECT DISTINCT 
			@sGlobalGroupUnq = G.GroupUnq, 
			@sGroupType = G.Type, 
			@sGlobalRotations = G.Rotations, 
			@sGlobalDateBegin = G.DateBegin, 
			@iGlobalNumDays = M.NumDays, 
			@sGlobalPosition = M.Position 
			FROM GroupMap M (NOLOCK), Groups G (NOLOCK) WHERE 
				M.AccountUnq = @iLoginAccountUnq AND 
				M.GroupUnq = G.GroupUnq AND 
				G.DomainUnq = @iDomainName AND 
				G.DomainUnq = M.DomainUnq
	END ELSE
	BEGIN
		/* Their Home Domain is not the current domain - do not log them in */
		SELECT @iLoginAccountUnq = -1
		SELECT @bHasAccount = 'N'
	END
END


Proc_Finalize:
/* Do we commit here or not */
IF @sInTransaction = 'Y' BEGIN
	IF @iRtnCode = 0
		COMMIT TRANSACTION
	ELSE
		ROLLBACK TRANSACTION
END

/* Create a recordset output if requested */
IF @sPrintMe = 'Y'
	SELECT @iRtnCode "RtnCode", @sGroupType "sGroupType", @sGlobalRotations "sGlobalRotations", @sGlobalDateBegin "sGlobalDateBegin", @sGlobalGroupUnq "sGlobalGroupUnq", @sGlobalPosition "sGlobalPosition", @iGlobalNumDays "iGlobalNumDays", @iLoginAccountUnq "iLoginAccountUnq", @bHasAccount "bHasAccount"

RETURN @iRtnCode